-- Priority List 3
-- by Jadya
-- EU-Well of Eternity

-- tab 5

local addonName, addonTable = ...
local L = addonTable.L
local T = addonTable.T
local AddColor = addonTable.AddColor

local addon = PriorityList
local wishlist = addon.WishListFunctions
local menus = addon.MenuFunctions
local items = addon.ItemFunctions
local comm  = addon.CommFunctions
local chars = addon.CharFunctions
local constructors = addon.Constructors

local dd_sender, dd_char
local sel_sender, sel_import_char

local function deleteSender()
 if not sel_sender or sel_sender == "" then return end
 addon.import_list[sel_sender] = nil
 addon:EventPulse("select_import_sender")
 addon:EventPulse("select_import_char")
 wipe(dd_char.elements)
end

local function clearSenders()
 wipe(addon.import_list)
 addon:Refresh()
end

local function importChar(name)
 if not name then return end
 
 PLWish_List[name] = nil
 chars:New(name)

 local imp = addon.import_list[sel_sender][name]
 for diff = 1,3 do
  if imp[diff].lists then PLWish_List[name][diff].lists  = imp[diff].lists end
  if imp[diff].looted then PLWish_List[name][diff].looted = imp[diff].looted end
 end
 addon.import_list[sel_sender][name] = nil
 local found
 for _ in pairs(addon.import_list[sel_sender]) do found = true break end
 if not found then
  addon.import_list[sel_sender] = nil
  sel_sender = ""
  sel_import_char = ""
  addon:EventPulse("select_import_char")
 end
end

local function importSender()
 if not sel_sender or sel_sender == "" then return end
 
 for k in pairs(addon.import_list[sel_sender]) do
  importChar(k)
 end
end

local function hasChanges(name)
 if not addon.import_list[sel_sender] or not addon.import_list[sel_sender][name] then return end
 
 local t = addon.import_list[sel_sender][name]

 if PLWish_List[name] then
  for diff = 1,3 do
   for list = 1,3 do
    -- lists
    local t2 = PLWish_List[name][diff].lists[list]
    for k,v in pairs(t[diff].lists[list]) do
     if t2 and v ~= t2[k] then
	  return AddColor("*", "LIGHT_GREEN")
	 end
    end
	-- looted
	t2 = PLWish_List[name][diff].looted
    for k,v in pairs(t[diff].looted) do
     if t2 and v ~= t2[k] then
	  return AddColor("*", "LIGHT_GREEN")
	 end
    end
   end
  end
 else
  return AddColor("*", "LIGHT_GREEN")
 end
 return ""
end

local function onShow()
 PLWLBtnList1:SetParent(PLTabFrame5)
 PLWLBtnList2:SetParent(PLTabFrame5)
 PLWLBtnList3:SetParent(PLTabFrame5)
 PLWLBtnList1:SetPoint("LEFT", dd_char, "RIGHT", 20, 0)
 PLListbox:SetMode("import")
 PLListbox:Show()
end

local function onHide()
 PLListbox:SetMode("normal")
end

local function updateListButton(self, itemID, looted)

 if not itemID then return false end

 local Iname, _, Iquality, _, _, _, _, _, IequipSlot, Itexture = items:GetItemInfo(itemID) -- import data
 local isWishlist, Wdifficulty, Windex, Wlooted = chars:FindItem(sel_import_char, itemID, addon.sel_difficulty)

 local _,_,_,c = GetItemQualityColor(Iquality or 1)

 local isImport
 local t = addon.import_list[sel_sender] and addon.import_list[sel_sender][sel_import_char]
 if t and t[addon.sel_difficulty] and t[addon.sel_difficulty].lists and t[addon.sel_difficulty].lists[addon.sel_list] then
  isImport = tContains(t[addon.sel_difficulty].lists[addon.sel_list], itemID)
 end
 self.t = "item"
 self.s_topright:SetText("")
 self.s_bottomleft:SetText("")
 self.s_bottomright:SetText("")
 self.s_center:SetText("")
 self.lefticon:SetTexture()
 self.righticon:SetTexture()
 self.centericon:Show()
 self.centericon:SetTexture()
 if not Iname then
  self.lefticon:SetTexture("Interface\\ICONS\\Ability_Mage_TimeWarp")
  self.s_topleft:SetText(AddColor(L["receiving_item_info"], "GOLD"))
 else
  if looted then
   self.link = looted.link
  elseif Wlooted then
   self.link = Wlooted.link
  else
   self.link = items:GenerateLink(itemID, Iname, Iquality, addon.sel_difficulty)
  end

  local s_looted = ""
  if looted and Wlooted and looted.link == Wlooted.link  then
   s_looted = AddColor(L["looted"], "GRAY")
  elseif looted and not Wlooted then
   s_looted = AddColor("+"..L["looted"], "LIGHT_GREEN")
  elseif not looted and Wlooted then
   s_looted = AddColor("-"..L["looted"], "LIGHT_RED")
  end

  if isWishlist and isImport then -- look for changes
   local import_index
   for k,v in pairs(t[addon.sel_difficulty].lists[addon.sel_list]) do
    if v == itemID then 
	 import_index = k
	 break
	end
   end
   if Windex > import_index then
    self.s_center:SetText(AddColor("+"..math.abs(Windex - import_index), "LIGHT_GREEN"))
	self.centericon:SetRotation(PI)
	self.centericon:SetTexture(T["arrowup"])
	self.centericon:Show()
   elseif Windex < import_index then
    self.s_center:SetText(AddColor("-"..math.abs(Windex - import_index), "LIGHT_RED"))
	self.centericon:SetRotation(0)
	self.centericon:SetTexture(T["arrowdown"])
	self.centericon:Show()
   else
    self.s_center:SetText("")
   end
   self.lefticon:SetTexture(Itexture)
   self.s_topleft:SetText((c and ("|c"..c) or "")..Iname.."|r")
   self.righticon:Hide()
   --self.righticon:SetTexture(Itexture)
   --self.s_topright:SetText((c and ("|c"..c) or "")..Iname.."|r")
   self.s_bottomleft:SetText(s_looted)
  elseif not isWishlist and isImport then -- to be added item
   self.lefticon:SetTexture(Itexture)
   self.righticon:SetTexture(T["added"])
   self.s_topleft:SetText((c and ("|c"..c) or "")..Iname.."|r")
   --self.s_topright:SetText(L["added"])
   self.s_center:SetText(L["added"])
   self.s_bottomleft:SetText(s_looted)
  else -- to be deleted item
   self.lefticon:SetTexture(T["deleted"])
   self.righticon:SetTexture(Itexture)
   self.s_topright:SetText((c and ("|c"..c) or "")..Iname.."|r")
   self.s_topleft:SetText(ACTION_SPELL_AURA_REMOVED)
   self.s_bottomright:SetText(s_looted)
  end
 end
 return true
end

function addon.listboxUpdate5(self)
 local offset  = HybridScrollFrame_GetOffset(self)
 local buttons = self.buttons
 local height  = 30

 local list = {}
 local looted
 -- add the imported items the list
 local t = addon.import_list[sel_sender] and addon.import_list[sel_sender][sel_import_char]
 if t and t[addon.sel_difficulty] and t[addon.sel_difficulty].lists[addon.sel_list] then
  for i = 1, #t[addon.sel_difficulty].lists[addon.sel_list] do
   list[i] = t[addon.sel_difficulty].lists[addon.sel_list][i]
  end
  looted = t[addon.sel_difficulty].looted
 end
 
 -- check for deleted items 
 if PLWish_List[sel_import_char] then
  for i = 1, #PLWish_List[sel_import_char][addon.sel_difficulty].lists[addon.sel_list] do
   if not tContains(list, PLWish_List[sel_import_char][addon.sel_difficulty].lists[addon.sel_list][i]) then
    table.insert(list, PLWish_List[sel_import_char][addon.sel_difficulty].lists[addon.sel_list][i])
   end
  end
 end
 
 if not list then
  self:HideButtons()
  return
 end

 for i = 1, #buttons do
  local index = offset + i
  if index <= #list and updateListButton(buttons[i], list[index], looted and looted[list[index]]) then
   buttons[i]:Show()
  else
   buttons[i]:Hide()
  end
 end
 
 HybridScrollFrame_Update(self, height * (list and #list or 0), height)
end

local function selectSender(sender, no_refresh)
 
 if not addon.import_list[sender] then
  sel_sender = ""
  dd_sender:SetText("")
  for k in pairs(addon.import_list) do
   selectSender(k)
   return
  end
  return
 end

 sel_sender = sender

 dd_sender:SetText(sender)

 dd_char.elements = addon.import_list[sel_sender]
 
 if not no_refresh then addon:Refresh() end
end

local function selectImportChar(name, no_refresh)
 
 if not addon.import_list[sel_sender] then
  sel_import_char = ""
  dd_char:SetText("")
  addon:Refresh() 
  return
 elseif not addon.import_list[sel_sender][name] then
  sel_import_char = ""
  dd_char:SetText("")
  for k in pairs(addon.import_list[sel_sender]) do
   selectImportChar(k)
   return
  end
  addon:Refresh() 
  return
 end
 sel_import_char = name

 dd_char:SetText(name)

 if not no_refresh then addon:Refresh() end
end

function addon.eventPulse5(event, ...)
 if event == "import_list" or event == "select_import_sender" then selectSender(...)
 elseif event == "select_import_char" then selectImportChar(...)
 end
end

function addon.createTab5()
 local f = CreateFrame("Frame", "PLTabFrame5", PLMainframe)
 f:SetPoint("TOPLEFT", PLMainframeTabAnchor, "TOPLEFT")
 f:SetPoint("BOTTOMRIGHT", PLMainframe, "BOTTOMRIGHT", -30, 60)
 f:SetScript("OnShow", onShow)
 f:SetScript("OnHide", onHide)
 
 -- import characters dropdown
 f = constructors:DropDown("PLDDImportSender", PLTabFrame5)
 f:SetPoint("TOPLEFT", PLTabFrame5, "TOPLEFT", 0, -40)
 f.lbl = f:CreateFontString()
 f.lbl:SetFont("Fonts\\ARIALN.ttf", 12, "")
 f.lbl:SetPoint("BOTTOM", f, "TOP", 0, 25)
 f.lbl:SetText(L["sender"])
 menus.Init_ImportSender(f)
 f.func_righttext = hasChanges
 f.isImport = true
 dd_sender = f
 
 -- import sender button
 f = constructors:RectButton("PLTab5ImportSenderBtn", PLTabFrame5)
 f:SetPoint("BOTTOM", dd_sender, "TOP", -25, 2)
 f:SetSize(40,20)
 f:SetText(L["import"])
 f:SetScript("OnClick", function()
  importSender()
  addon:EventPulse("select_import_sender")
 end)
 
 
 -- delete sender button
 f = constructors:RectButton("PLTab5DelSenderBtn", PLTabFrame5)
 f:SetPoint("BOTTOM", dd_sender, "TOP", 25, 2)
 f:SetSize(40,20)
 f:SetText(CALENDAR_DELETE_EVENT)
 f:SetScript("OnClick", function()
  deleteSender()
  addon:EventPulse("select_import_sender")
 end)

 -- import characters dropdown
 f = constructors:DropDown("PLDDImportChar", PLTabFrame5)
 f:SetPoint("LEFT", dd_sender, "RIGHT", 0, 0)
 f.lbl = f:CreateFontString()
 f.lbl:SetFont("Fonts\\ARIALN.ttf", 12, "")
 f.lbl:SetPoint("BOTTOM", f, "TOP", 0, 25)
 f.lbl:SetText(CHARACTER)
 menus.Init_ImportChar(f)
 f.isImport = true
 dd_char = f
 
 -- import char button
 f = constructors:RectButton("PLTab5ClearSenderBtn", PLTabFrame5)
 f:SetPoint("BOTTOM", dd_char, "TOP")
 f:SetSize(40,20)
 f:SetText(L["import"])
 f:SetScript("OnClick", function()
  importChar(sel_import_char)
  addon:EventPulse("select_import_char")
 end)
 
 PLTabFrame5:Hide()
end