﻿-- Priority List 3
-- by Jadya
-- EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L
local C = addonTable.C
local addon = PriorityList

local AddColor = addonTable.AddColor

local items = addon.ItemFunctions

local waiting_itemIDs = {} -- get_item_info queries

addon.opt = {}

function items:ExtractItemID(link)
 local _, _, itemID = strfind(link, "item:(%d+):")
 return tonumber(itemID)
end

function items:ExtractBonuses(link)

 local result = {}
 
 local i,j = strfind(link, ":(%d+)|h")
 link = strsub(link, 1, j - 2)
 local _, _, _, _, _, _, _, _, _, _, _, _, _, numBonusIDs = strsplit(":", link)
 if tonumber(numBonusIDs) > 0 then
  local bonuses = {select(15, strsplit(":", link))}
  for k,v in pairs(bonuses) do
   v = tonumber(v)
   for i = 1,C.numBonuses do
    if not result[i] and tContains(C.bIDs[i], v) then
	 result[i] = true
	end
    end
  end
 end

 return result -- wf, socket, avoidance, leech, speed
end

function items:GetBonusString(bonuses, short)
 if not bonuses then return "" end

 local result = ""
 for i = 1,C.numBonuses do
  if bonuses[i] then
   result = result.." "..L[(short and "bonus_short" or "bonus")..i]
  end
 end
 return result
end

function items:GetBonusScore(bonuses)
 local primary, secondary = 0, 0
 for i = 1, 2 do
  primary = primary + (bonuses[i] and pl_options["bonus_value_"..i] or 0)
 end
 for i = 3, C.numBonuses do
  secondary = secondary + (bonuses[i] and pl_options["bonus_value_"..i] or 0)
 end
 return primary, secondary
 
--[[
 local result = 0
 for i = 1,C.numBonuses do
  result = result + (bonuses[i] and pl_options["bonus_value_"..i] or 0)
 end
 return result
 ]]
end

function items:GetDifficulty(link)
 local difficultyID = select(13, strsplit(":", link))
 -- normal 3, heroic 5, mythic 6
 return (difficultyID == "5" or difficultyID == "6") and difficultyID - 3 or 1
end

function items:GenerateLink(itemID, name, quality, difficulty, bonuses)
 if not quality then
  name, _, quality = GetItemInfo(itemID)
 end

 local _, _, _, col = GetItemQualityColor(quality or 1)
 local diff = difficulty == 1 and 3 or (difficulty or 0) + 3

 local numBonus = 0
 local s_bonus = ""
 local function addBonus(v)
  numBonus = numBonus + 1
  s_bonus = s_bonus..":"..v
 end
 
 if     diff == 5 then addBonus(566) -- heroic +15
 elseif diff == 6 then addBonus(567) -- mythic +30
 end

 if bonuses then
  for i = 1,C.numBonuses do
   if bonuses[i] then
    addBonus(C.bIDs[i][1])
   end
  end
 end

 return "|c"..col.."|Hitem:"..itemID..":0:0:0:0:0:0:0:100:268:0:"..diff..":"..numBonus..s_bonus.."|h["..(name or "PL3 Item").."]|h|r"
end

function items:GetItemInfo(itemID)
 local name, link, quality, _, _, _, _, _, equipSlot, texture = GetItemInfo(itemID)
 
 if not link then
  waiting_itemIDs[itemID] = true
 end
 
 return name, link, quality, _, _, _, _, _, equipSlot, texture
end

local timeout = 0
local timer = CreateFrame("Frame")
timer:SetScript("OnUpdate", function(self) 
 if timeout <= time() then
  addon:Refresh()
  self:Hide()
 end
end)
timer:Hide()

addon.eventhandler:RegisterEvent("GET_ITEM_INFO_RECEIVED")
function addon.eventhandler.GET_ITEM_INFO_RECEIVED(...)
 local itemID = ...
 if waiting_itemIDs[itemID] and GetItemInfo(itemID) then
  waiting_itemIDs[itemID] = nil
  timeout = time() + 2
  timer:Show()
 end
end